import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt


def tiempo_a_segundos(tiempo):
    if(len(tiempo.split('m')) > 1):
        minutos, segundos = tiempo.split('m')
        minutos = int(minutos)
        segundos = float(segundos.rstrip('s'))
        total_segundos = minutos * 60 + segundos
        return total_segundos
    else:
        return 0.0

df = pd.read_csv('tiempos.csv')

filtered = df.groupby(["programa", "ejemplo"])["tiempo"].mean()

prueba = filtered.reset_index()

prueba['tiempo'] = prueba['tiempo'].round(4)

prueba = prueba.sort_values(by="ejemplo")

prueba = prueba.pivot(index='ejemplo', columns='programa', values='tiempo')

prueba = prueba.reset_index()

prueba = prueba[['ejemplo', 'prolog', 'python', 'optimized']]

prueba = prueba.replace(120.0, 3600.0)



prueba.to_csv('filtrado.txt', index=False)


sns.set(style='whitegrid')

# #Gráfica completa

prolog_without_timeout = prueba[prueba['prolog'] != 0.0]
python_without_timeout = prueba[prueba['python'] != 0.0]
optimized_without_timeout = prueba[prueba['optimized'] != 0.0]

#Sen Heurística vs optimized


fig4 = plt.figure(figsize=(10,8))

ax4 = fig4.add_subplot(1,1,1)

ax4.set_title('Sen Heurística vs Con Heurística')

sns.scatterplot(x='optimized', y='python', data=python_without_timeout)

ax4.set_xlim(left=0,right=3600)
ax4.set_ylim(bottom = 0, top=3600)

plt.xlabel('Con Heurística')
plt.ylabel("Sen Heurística")

plt.xticks(rotation=90)

#

fig3 = plt.figure(figsize=(10,8))

ax3 = fig3.add_subplot(1,1,1)

ax3.set_title('Prolog vs Sen Heurística')

sns.scatterplot(x='prolog', y='python', data=python_without_timeout)

ax3.set_xlim(left=0,right=3600)
ax3.set_ylim(bottom = 0, top=3600)

plt.xlabel('Prolog')
plt.ylabel("Sen Heurística")

plt.xticks(rotation=90)

#

fig5 = plt.figure(figsize=(10,8))

ax5 = fig5.add_subplot(1,1,1)

ax5.set_title('Prolog vs Sen Heurística')

sns.scatterplot(x='prolog', y='python', data=python_without_timeout)

ax5.set_xlim(left=0,right=500)
ax5.set_ylim(bottom = 0, top=500)

plt.xlabel('Prolog')
plt.ylabel("Sen Heurística")

plt.xticks(rotation=90)

#

fig2 = plt.figure(figsize=(10,8))

ax2 = fig2.add_subplot(1,1,1)

ax2.set_title('Prolog vs Con heurística')

sns.scatterplot(x='prolog', y='optimized', data=python_without_timeout)

ax2.set_xlim(left=0,right=3600)
ax2.set_ylim(bottom = 0, top=3600)

plt.xlabel('Prolog')
plt.ylabel("Con heurística")

plt.xticks(rotation=90)

#

fig6 = plt.figure(figsize=(10,8))

ax6 = fig6.add_subplot(1,1,1)

ax6.set_title('Prolog vs Con heurística')

sns.scatterplot(x='prolog', y='optimized', data=python_without_timeout)

ax6.set_xlim(left=0,right=500)
ax6.set_ylim(bottom = 0, top=500)

plt.xlabel('Prolog')
plt.ylabel("Con heurística")

plt.xticks(rotation=90)

#

re = pd.read_csv('dt.csv')
re['optimized'] = re['optimized'].apply(tiempo_a_segundos)
re['python'] = re['python'].apply(tiempo_a_segundos)

fig = plt.figure(figsize=(10,8))

ax = fig.add_subplot(1,1,1)

ax.set_title('Árboles reales')

sns.scatterplot(x='python', y='optimized', data=re)

ax.set_xlim(left=0,right=3600)
ax.set_ylim(bottom = 0, top = 3600)

plt.xlabel('python')
plt.ylabel("optimized")

plt.xticks(rotation=90)

# Mostrar el gráfico
plt.show()


from Implicant.intervalmplicant import *

class optimizedIntervalImplicant(IntervalImplicant):
    def unfold(self, variableStage):
        newSteps = []

        for var in variableStage:
            newSteps.append(list(range(0, len(var))))

        unfolds = self.unfoldTuple(self.list, variableStage, newSteps)
        result = set()

        for imp in unfolds:
            result.add(IntervalImplicant(imp))

        return result
    
    def unfoldTuple(self, tuple, variableStage, newSteps):
        if(tuple==[]):
            yield []
        else:
            start = tuple[0][0]
            end = tuple[0][1]
            steps = variableStage[0]
            normalized = newSteps[0]
            i = steps.index(start)

            while steps[i]!=end:
                for imp in self.unfoldTuple(tuple[1:], variableStage[1:], newSteps[1:]):
                    yield [(normalized[i], normalized[i+1])]+imp
                i=i+1


import math
import random
from Implicant.intervalmplicant import *

class IntervalsGenerator(object):
    @classmethod
    def generate(self, nVars = 3, maxnThres = 15, nImpl = 14):
        N = 100

        stagesList = []
        implicantList = []

        maxStages = 0

        for _ in range (nVars):
            numStages = random.randint(2, maxnThres)

            stages = list(range(0, N+1, math.ceil(N/numStages)))

            if(len(stages)==numStages):
                stages.append(100)
            stagesList.append(stages)

            if(len(stages)-1 > maxStages):
                maxStages = len(stages)-1

        self.stages = stagesList
        self.maxStages = maxStages

        for _ in range(nImpl):
            implicant = []
            for v in range (nVars):
                sample = random.sample(stagesList[v], 2)
                implicant.append((min(sample), max(sample)))
            implicantList.append(IntervalImplicant(implicant))

        return implicantList


from Implicant.implicant import *

class IntervalImplicant(Implicant):
    def __init__(self, list):
        self.list = list

    def matches(self, otherImplicant):

        difference = 0
        merging = []

        for v in range(len(self.list)):
                    a, b = self.list[v]
                    c, d = otherImplicant.list[v]

                    if b >= c and a <= d:
                        interval = (min(a,c), max(b, d))
                        merging.append(interval)
                        if a != c or b != d:
                            difference += 1
                    else:
                        return None

        if difference == 1:
            return IntervalImplicant(merging)
    
        return None
    
    def unfold(self, variableStage):
        unfolds = self.unfoldTuple(self.list, variableStage)
        result = set()

        for imp in unfolds:
            result.add(IntervalImplicant(imp))

        return result
    
    def unfoldTuple(self, tuple, variableStage):
        if(tuple==[]):
            yield []
        else:
            start = tuple[0][0]
            end = tuple[0][1]
            steps = variableStage[0]
            i = steps.index(start)

            while steps[i]!=end:
                for imp in self.unfoldTuple(tuple[1:], variableStage[1:]):
                    yield [(steps[i], steps[i+1])]+imp
                i=i+1
    
    def __str__ (self):
        return f'{str(self.list)}'

    def __hash__ (self):
        return hash(str(self.list))

    def __eq__ (self, other) -> bool:
        return self.list == other.list


from Resources.resources import *

class QuineMcCluskey(object):
    @classmethod
    def execute(self, input):

        output = set()

        while(input):
            auxInput = {element: 0 for element in input}
            mergingSet = set()

            for i in range(0, len(input)-1):
                for j in range(i+1, len(input)):
                    first = input[i]
                    second = input[j]

                    fusion = first.matches(second)
                    
                    if fusion is not None:
                        mergingSet.add(fusion)
                        auxInput[first]=1
                        auxInput[second]=1

            output.update(Resources.primeImplicants(auxInput))
            
            input = list(mergingSet)

        return output


# import sys
# import os
# import csv
# import pandas as pd

# with open("prologTimes.txt") as f:
#     prolog = f.read().splitlines() 

# with open("pythonTimes.txt") as f:
#     python = f.read().splitlines() 

# times = dict()

# for l in prolog:
#     splitted = l.split(".")
#     seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
#     if times.get(splitted[1]) is None:
#         times.update({splitted[1]: [seconds]})
#     else:
#         times[splitted[1]].append(seconds)

# for l in python:
#     splitted = l.split(".")
#     seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
#     times[splitted[1]].append(seconds)

# tiempos = list(times.items())

# with open('tiempos.csv', mode='w', newline='') as file:
#     writer = csv.writer(file)
#     writer.writerow(["ejecución", "programa", "ejemplo", "tiempo"])
#     for t in tiempos:
#         for i in range(0, 5):
#             writer.writerow([i+1, "prolog", t[0], t[1][i]])
#             writer.writerow([i+1, "python", t[0], t[1][i+5]])

# file.close()

# df = pd.read_csv('tiempos.csv')
# filtered = df.filter(items = ['programa'])

# print(filtered.to_string())

import sys
import os
import csv
import copy


def main():

    prologTimes = {}

    if(len(sys.argv) < 2):
        print("Please give a test folder")
        return
    
    files = os.listdir('tests/'+sys.argv[1]+'/GeneratedExamples')

    prologTimes = {}

    for file in files:
        splitted = file.split('.')
        prologTimes.update({splitted[0]: []})

    pythonTimes = copy.deepcopy(prologTimes)
    optimizationTimes = copy.deepcopy(prologTimes)

    with open("tests/"+sys.argv[1]+"/times/prologTimes.txt") as f:
       prolog = f.read().splitlines() 

    with open("tests/"+sys.argv[1]+"/times/pythonTimes.txt") as f:
        python = f.read().splitlines()

    with open("tests/"+sys.argv[1]+"/times/optimizedTimes.txt") as f:
        optimized = f.read().splitlines()

    with open("tests/"+sys.argv[1]+"/times/fc.txt") as f:
        calculatefc = f.read().splitlines()

    values = dict()

    for l in calculatefc:
        splitted = l.split(",")
        example = splitted[0].split('.')[0]
        fc = splitted[1]
        mfc = splitted[2]
        values[example] = [fc, mfc]

    for l in prolog:
        splitted = l.split(".")
        if splitted[2]!= '':
            minutes = splitted[2].split("m")[0]
            seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
            time = (int(minutes) * 60) + float(seconds)
            print(splitted[0], minutes, seconds, time)
        else:
            time = 120
            
        prologTimes[splitted[0]].append(time)

    for l in python:
        splitted = l.split(".")
        if splitted[2]!= '':
            minutes = splitted[2].split("m")[0]
            seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
            time = (int(minutes) * 60) + float(seconds)
            print(splitted[0], minutes, seconds, time)
        else:
            time = 120

        pythonTimes[splitted[0]].append(time)

    for l in optimized:
        splitted = l.split(".")
        print(splitted)
        if splitted[2]!= '':
            minutes = splitted[2].split("m")[0]
            seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
            time = (int(minutes) * 60) + float(seconds)
            print(splitted[0], minutes, seconds, time)
        else:
            time = 120

        optimizationTimes[splitted[0]].append(time)

    prologList = list(prologTimes.items())
    pythonList = list(pythonTimes.items())
    optimizationList = list(optimizationTimes.items())

    with open("tests/"+sys.argv[1]+"/times/tiempos.csv", mode='w', newline='') as file:
        writer = csv.writer(file)
        writer.writerow(["programa", "fc", "mfc", "ejemplo", "tiempo"])
        for t in prologList:
            if len(t[1]) != 0:
                writer.writerow(["prolog", values.get(t[0])[0], values.get(t[0])[1], t[0], t[1][0]])

        for p in pythonList:
            if len(t[1]) != 0:
                writer.writerow(["python", values.get(p[0])[0], values.get(p[0])[1], p[0], p[1][0]])

        for o in optimizationList:
            if len(t[1]) != 0:
                writer.writerow(["optimized", values.get(o[0])[0], values.get(o[0])[1], o[0], o[1][0]])

    file.close()


if __name__ == "__main__":
    main()


import sys
from Resources.intervalsGenerator import *

def main():
    arguments = sys.argv

    steps = [int(i) for i in sys.argv[1].strip('][').split(',')]

    implicants = [int(i) for i in sys.argv[2].strip('][').split(',')]

    vars = [int(i) for i in sys.argv[3].strip('][').split(',')]

    #print(steps, implicants, vars)

    for s in steps:
        for i in implicants:
            for v in vars:
                generator = IntervalsGenerator()
                generatedImplicants = generator.generate(v, s, i)

                filename = "ex." + str(v) + "_" + str(s) + "_" + str(i)

                with open('GeneratedExamples/' + filename, 'w') as f:
                    f.write(str(v) + '\n')

                    for stage in generator.stages:
                        f.write(str(max(stage)) + "\n")

                    for implicant in generatedImplicants:
                        for tuple in implicant.list:
                            for number in tuple:
                                f.write(str(number) + " ")
                        f.write("\n")

                #print("\nSteps: ",s, " Vars: ", v, " Imlicants: ", i, "Filename: ", filename)
                #print("numStages: ", generator.maxStages, "stages:", generator.stages)
                #print("implicants: ", '\n'.join(map(str, generatedImplicants)))



if __name__ == "__main__":
    main()


from Resources.readInput import *
from Resources.intervalsGenerator import *
from quineMcCluskey import *
import sys

def main():

    readInput = ReadInput()

    input = readInput.readIntervalInput('tests/'+sys.argv[1]+'/GeneratedExamples/' + sys.argv[2])

    newImplicants = set()

    for element in input:
        newImplicants.update(element.unfold(readInput.stages))

    staggeredImplicants = list(newImplicants)

    #print("StaggeredImplicants: ", staggeredImplicants)

    output = QuineMcCluskey.execute(staggeredImplicants)

    print(output)
    
    filename = sys.argv[2]

    with open('tests/'+sys.argv[1]+'/PyOutput/' + filename, 'w') as f:
        for implicant in list(output):
            for tuple in implicant.list:
                    for number in tuple:
                        f.write(str(number) + " ")
            f.write("\n")


if __name__ == "__main__":
    main()


from Resources.readInput import *
from Resources.intervalsGenerator import *
from quineMcCluskey import *
import sys

#Recibimos un archivo input
#Lo leemos y tenemos nuestra representación en intervalos
#Ejecutamos el algoritmo
#Devolvemos la salida, se puede guardar en un archivo

class DtMinimisation():
    def __init__(self, path):
        readInput = ReadInput()

        input = readInput.readIntervalInput(path)

        newImplicants = set()

        for element in input:
            newImplicants.update(element.unfold(readInput.stages))

        staggeredImplicants = list(newImplicants)

        output = QuineMcCluskey.execute(staggeredImplicants)

        self.input = input
        self.staggeredImplicants = staggeredImplicants
        self.output = output

    def printInput(self):
        print("Input implicants for the Decision Tree minimizer:\n")
        for implicant in self.input:
            print(implicant)
        print("\n")

    def printStaggeredInput(self):
        print("Staggered implicants from the input:\n")
        for implicant in self.staggeredImplicants:
            print(implicant)
        print("\n")

    def printOutput(self):
        print("Ouput implicants:\n")
        for implicant in self.output:
            print(implicant)
        print("\n")



import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

df = pd.read_csv('tiempos.csv')
filtered = df.groupby(["programa", "ejemplo"])["tiempo"].mean()

prueba = filtered.reset_index()

prueba['tiempo'] = prueba['tiempo'].round(4)

prueba = prueba.sort_values(by="ejemplo")

prueba = prueba.pivot(index='ejemplo', columns='programa', values='tiempo')

prueba = prueba.reset_index()

prueba = prueba[['ejemplo', 'prolog', 'python', 'optimized']]

prueba = prueba.replace(120.0, 0.0)

print(prueba)



prueba.to_csv('filtrado.txt', index=False)


sns.set(style='whitegrid')

prolog_without_timeout = prueba[prueba['prolog'] != 0.0]
sns.scatterplot(x='ejemplo', y='prolog', data=prolog_without_timeout, label='prolog')

python_without_timeout = prueba[prueba['python'] != 0.0]
sns.scatterplot(x='ejemplo', y='python', data=python_without_timeout, label='python')

optimized_without_timeout = prueba[prueba['optimized'] != 0.0]
sns.scatterplot(x='ejemplo', y='optimized', data=optimized_without_timeout, label='optimized')

plt.xticks(rotation=90)

plt.legend()

# Mostrar el gráfico
plt.show()


from Resources.readInput import *
from Resources.intervalsGenerator import *
from quineMcCluskey import *
import sys

def main():
    if(len(sys.argv) < 3):
        print("Please give a testFolder")
        return

    readInput = ReadInput()

    input = readInput.readIntervalInput('tests/'+sys.argv[1]+'/GeneratedExamples/' + sys.argv[2]) 

    newImplicants = set()

    for element in input:
        newImplicants.update(element.unfold(readInput.stages))

    MFC = 1

    for variableStage in readInput.stages:
        MFC = MFC * (len(variableStage)-1)

    with open('tests/'+sys.argv[1]+'/times/fc.txt', 'a') as f:
        f.write(sys.argv[2] + ',' + str(len(newImplicants)) + ',' + str(MFC)+'\n')

    print(readInput.stages)
    print("MFC number is ", MFC)
    print("FC number is ", len(newImplicants))
    print("Covered lines ", (len(newImplicants)/MFC)*100, "%")


if __name__ == "__main__":
    main()